<?php // Please do not edit this file. If you do, save as UTF-8 without BOM!

/* Copyright 2010-2022 Martin Janecke <aneamal@prlbr.de>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

// declare (strict_types = 1); // only during development

namespace prlbr\aneamal;

// error_reporting (E_ALL); // only during development

// Constants
const vmain = '29';
const CACHE_READ = 1;
const CACHE_FEED = 2;
const CACHE_DROP = 4;

// filename
$query = $_SERVER['QUERY_STRING'];
$file  = __DIR__ . '/..' . $query;
$base  = basename ($query);

header ('Content-Type: text/html; charset=UTF-8');

// Restrict further processing to readable .nml files. The file extentsion check
// prevents sensitive data such as in .htaccess or .php from becoming public.
if (substr ($base, -4) !== '.nml' or !is_readable ($file)):
	header ('HTTP/1.1 404 Not Found');
	print <<<ANT
		<!doctype html>
		<title>404 Not Found</title>
		<h1>Not Found</h1>
		<p>The requested URL was not found on this server.</p>
		ANT;
	exit;
endif;

// Aneamal file names with an initial @-symbol are for automatic inclusion in
// other pages and not made to be accessed directly, hence the 403 status. They
// are still processed normally when accessed directly for trouble shooting.
if ($forbidden = $base[0] === '@'):
	header ('HTTP/1.1 403 Forbidden');
endif;

// Determine the caching mode. AneamalCache 'on' means use cache except for
// HTTP POST requests; 'forced' uses the cache independent of HTTP method; an
// integer is like 'on' with a maximum usability period given in seconds.
$mode = 0;
if ($rule = getenv ('AneamalCache')):
	$rule = strtolower ($rule);
	$hash = md5 ($query);
	$hash[2] = '/';
	$cached = __DIR__ . '/private/cache/' . $hash . '.html';
	if ($_SERVER['REQUEST_METHOD'] === 'POST' and $rule !== 'forced'):
		file_exists ($cached) and $mode = CACHE_DROP;
	elseif ($time = @filemtime ($cached)): // checks existence and time
		if ($rule === 'on' or $rule === 'forced'):
			$mode = $time > filemtime ($file)? CACHE_READ: CACHE_FEED;
		elseif ($life = (int) $rule):
			$mode = $time > filemtime ($file) && $time + $life > time ()? CACHE_READ: CACHE_FEED;
		endif;
	elseif ($rule === 'on' or $rule === 'forced' or (int) $rule):
		$mode = CACHE_FEED;
	endif;
endif;

// Get and output the page from the cache.
if ($mode === CACHE_READ):

	// Implements https://tools.ietf.org/html/rfc7232#section-2.2
	header ('Last-Modified: ' . gmdate ('D, d M Y H:i:s', $time) . ' GMT');

	// Implements https://tools.ietf.org/html/rfc7232#section-3.3
	if (
		isset ($_SERVER['HTTP_IF_MODIFIED_SINCE'])
		and $_SERVER['REQUEST_METHOD'] === 'GET' || $_SERVER['REQUEST_METHOD'] === 'HEAD'
		and @strtotime ($_SERVER['HTTP_IF_MODIFIED_SINCE']) >= $time
		and !$forbidden
	):
		header ('HTTP/1.1 304 Not Modified');
		exit;
	elseif (@readfile ($cached)):
		exit;
	endif;
endif;

// start timing of page generation
$start = microtime (true);

// include the class file which does the Aneamal-HTML-translation
require __DIR__ . '/html.php';

// get the Aneamal root directory relative to the document root; the 17
// removed characters from the SCRIPT_NAME are "/aneamal/main.php"
$home = getenv ('AneamalHome') ?: substr ($_SERVER['SCRIPT_NAME'], 0, -17);

// create an object doing the Aneamal-HTML-translation and pass the
// Aneamal file contents to it
$translator = new nml2html (file_get_contents ($file), dirname ($query), $home, $base);
$html = $translator->document ();

// stop timing
$duration = microtime (true) - $start;

// print the HTML page
print $html . '<!--' . strval ($duration) . '-->';

// write the file to the cache, create directory if necessary
if ($mode === CACHE_FEED):
	if (@file_put_contents ($cached, $html . '<!--' . $hash . '-from-cache-->')):
		print '<!--written-to-cache-->';
	elseif (is_dir ($dir = dirname ($cached)) or !@mkdir ($dir, 0777, true)):
		print '<!--failed-to-cache-->';
	elseif (@file_put_contents ($cached, $html . '<!--' . $hash . '-from-cache-->')):
		print '<!--written-to-cache-->';
	else:
		print '<!--failed-to-cache-->';
	endif;
elseif ($mode === CACHE_DROP):
	touch ($cached, 0);
endif;

// The End